/*
 * Copyright (c) 2012-2017 Andes Technology Corporation
 * All rights reserved.
 *
 */

#ifndef __AE250_H__
#define __AE250_H__

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
 * System clock
 ****************************************************************************/
#define KHz                     1000
#define MHz                     1000000

#define OSCFREQ                 (20 * MHz)
#define CPUFREQ                 (40 * MHz)
#define RTCFREQ                 (32768)
#define HCLKFREQ                (CPUFREQ)
#define PCLKFREQ                (CPUFREQ)
#define UCLKFREQ                (OSCFREQ)

/*****************************************************************************
 * PLIC Interrupt source definition
 ****************************************************************************/
#define IRQ_RTCPERIOD_SOURCE    1
#define IRQ_RTCALARM_SOURCE     2
#define IRQ_PIT_SOURCE          3
#define IRQ_SPI1_SOURCE         4
#define IRQ_SPI2_SOURCE         5
#define IRQ_I2C_SOURCE          6
#define IRQ_GPIO_SOURCE         7
#define IRQ_UART1_SOURCE        8
#define IRQ_UART2_SOURCE        9
#define IRQ_DMA_SOURCE          10
#define IRQ_BMC_SOURCE          11
#define IRQ_SWINT_SOURCE        12
#define IRQ_SDC_SOURCE          18
#define IRQ_MAC_SOURCE          19
#define IRQ_HW_INT_1	        20
#define IRQ_HW_INT_2  		    21
#define IRQ_HW_INT_3        	22
#define IRQ_HW_INT_4        	23
#define IRQ_HW_INT_5        	24
#define IRQ_HW_INT_6        	25
#define IRQ_STANDBY_SOURCE      26
#define IRQ_WAKEUP_SOURCE       27

#ifndef __ASSEMBLER__

/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/

#define __I                     volatile const	/* 'read only' permissions      */
#define __O                     volatile        /* 'write only' permissions     */
#define __IO                    volatile        /* 'read / write' permissions   */

/*****************************************************************************
 * PLMT - AE250
 ****************************************************************************/
typedef struct {
	__IO unsigned long long MTIME;          /* 0x00 Machine Time */
	__IO unsigned long long MTIMECMP;       /* 0x08 Machine Time Compare */
} PLMT_RegDef;


/*****************************************************************************
 * Memory Map
 ****************************************************************************/

#define _IO_(addr)              (addr)

#define EILM_BASE               0x00000000
#define EDLM_BASE               0x00200000
//#define SPIMEM_BASE             0x80000000

#define PLIC_BASE               _IO_(0xE4000000)
#define PLMT_BASE               _IO_(0xE6000000)
#define PLIC_SW_BASE            _IO_(0xE6400000)
#define PLDM_BASE               _IO_(0xE6800000)

/*****************************************************************************
 * Peripheral device declaration
 ****************************************************************************/
#define AE250_PLMT              ((PLMT_RegDef *) PLMT_BASE)


#endif	/* __ASSEMBLER__ */

#ifdef __cplusplus
}
#endif

#endif	/* __AE250_H__ */
